#include "print.h"

int log_fd=-1;

int print_init(void)
{
	if ( flag(FLAG_STDOUT) && flag(FLAG_DAEMON) ){
		flag_unset(FLAG_STDOUT);
	}
	
	if ( flag(FLAG_LOGFILE) ){
		if ( !globals.logfile || !*(globals.logfile) ) {
			print_err(ERR_OPEN_LOGFILE CRLF, globals.logfile);
			return 0;
		}
		
		log_fd = open(globals.logfile, O_WRONLY | O_APPEND | O_CREAT, 00600);
		if ( log_fd < 0 ) {
			print_err(ERR_OPEN_LOGFILE CRLF, globals.logfile);
			return 0;
		}
	}

	return 1;
}

void print_close(void)
{
	if ( flag(FLAG_LOGFILE) ){
		if ( log_fd >= 0 ) close(log_fd);
	}
}

void print_dump ( p_packet p )
{
	int sport=0, dport=0;
	char sip[BUFFER_SIZE], dip[BUFFER_SIZE];
	char *source_ip=(char *)&sip, *dest_ip=(char *)&dip;
	char tb[512];

	/* basic header types */
	struct tcphdr  *tcp = p->trans;
	struct udphdr  *udp = p->trans;
	struct icmphdr *icmp = p->trans;

	bzero(source_ip, BUFFER_SIZE);
	bzero(dest_ip, BUFFER_SIZE);

	format_ip( p->ip->saddr, source_ip);
	format_ip( p->ip->daddr, dest_ip);

	/* set ports, dependant on protocol */
	switch ( p->ip->protocol ){
		case IPPROTO_TCP:			
			sport= ntohs(tcp->source);
			dport= ntohs(tcp->dest);
			break;
		case IPPROTO_UDP:
			sport=ntohs(udp->source);
			dport=ntohs(udp->dest);
			break;
		case IPPROTO_ICMP:
			sport=icmp->type;
			dport=icmp->code;
			break;
	}

	/* Format the time */
	strftime((char *)&tb, 512, flag(FLAG_SHOWDATE) ? "%d/%m/%y %H:%M:%S" :
		"%H:%M:%S", p->time);
		
	globals.nosync = 1;
	
	/* date/info */
	print_out("%s ", (char *)&tb);
	print_out ( "[%s] %s %s:%i -> %s:%i", p->data.netlink.iface, get_proto(p->ip->protocol),
					source_ip, sport, dest_ip, dport);

	/* print proto specific stuff */
	switch ( p->ip->protocol ){
		case IPPROTO_TCP:
			print_out(" [");
			if ( tcp->ack ) print_out("A");
			if ( tcp->psh ) print_out("P");
			if ( tcp->rst ) print_out("R");
			if ( tcp->syn ) print_out("S");
			if ( tcp->fin ) print_out("F");
			print_out("]");
			break;
		case IPPROTO_ICMP:
			print_out (" ttl=%i ", p->ip->ttl);
			print_icmp_type(icmp->type, icmp->code);
			break;
	}

	/* print the fragmentation offset, if its a non-first fragment */
	//if (p->ip->frag_off)
	//	print_out ( " frag=0x%04X", p->ip->frag_off);

	if ( !flag(FLAG_NODATA) ) {
		hex_dump( p );
	}
	
	print_out ( CRLF );
	print_sync();
	globals.nosync = 0;
}

void print_sync(void)
{
	/* Write to stdout */
	if ( flag(FLAG_STDOUT) && !flag(FLAG_DAEMON) ) fsync(0);

	/* Write to logfile */
	if ( flag(FLAG_LOGFILE) && log_fd >= 0 ) fsync(log_fd);
}

/* output function */
void print_out(char *format, ...)
{
	va_list va;
	int len;
	char output[BUFFER_SIZE];
	char *p=(char *)&output;
	bzero(p, BUFFER_SIZE);
	
	// format the data
	va_start(va, format);
	vsprintf(p, format, va);
	va_end(va);

	len = strlen(p);

	/* Write to stdout */
	if ( flag(FLAG_STDOUT) && !flag(FLAG_DAEMON) ) {
		write(0, p, len);
		if ( !globals.nosync ) fsync(0);
	}

	/* Write to logfile */
	if ( flag(FLAG_LOGFILE) && log_fd >= 0 ){
		write(log_fd, p, len);
		if ( !globals.nosync ) fsync(log_fd);
	}
}

void hex_dump( p_packet p )
{
	int i,j=0;
	int line;
	int plen=ntohs(p->ip->tot_len);
	unsigned char *tmp;

	tmp=(unsigned char*)p->ip;
	print_out ( " len=%d", ntohs(p->ip->tot_len) );	
	print_out ( CRLF );	


	while ( j < plen ) {
		/* work out length of line */
		if ( (j+globals.length) > plen )
			line = plen-j;
		else
			line = globals.length;

		/* print offset */
		print_out("%04X : ", j);

		if ( !flag(FLAG_NOASCII) ) {	
			for ( i=0; i<globals.length; i++ ) {
				if ( i<line ) {
					if ( isprint(tmp[i]) )
						print_out("%c", tmp[i]);
					else
						print_out(".");
				}else{
					print_out(".");
				}
			}
		}
		
		if ( !flag(FLAG_NOHEX) ) {
			for ( i=0; i<line; i++ ) {
					print_out(" %02X", tmp[i]);
			}
		}

		j+=line;
		tmp+=line;
		print_out(CRLF);
	}

}
